from PIL import Image
import qrcode

from .error import *

def load_image(fname):
    im = Image.open(fname)
    (w, h) = im.size
    if w != h:
        raise ImageNotSquareException
    im_ = im.convert("RGBA")
    return im_

# FIXME: slow
def rgba_search_replace(img, old, new):
    for x in range(0, img.size[0]):
        for y in range(0, img.size[1]):
            px = img.getpixel((x, y))
            if px == old:
                img.putpixel((x, y), new)

def make_qr(data,
            box_size,
            border,
            width,
            height,
            qr_opacity=200,
            white_opacity=0,
            qr_rgb=(0,0,0)):
    qr = qrcode.QRCode(
        version=None, # pick version based on the data
        error_correction=qrcode.constants.ERROR_CORRECT_H,
        box_size=box_size,
        border=0, # we make our own border
    )
    qr.add_data(data)
    qr.make(fit=True)
    qr_img = qr.make_image().convert("RGBA")

    # Shrinking QR code is not gonna work at one pixel per module.
    if qr_img.size[0] > width or qr_img.size[1] > height:
        raise BaseImageSmallerThanQRCodeException

    rgba_search_replace(qr_img,
                        (255, 255, 255, 255),
                        (255, 255, 255, white_opacity))
    rgba_search_replace(qr_img,
                        (0, 0, 0, 255),
                        (qr_rgb[0], qr_rgb[1], qr_rgb[2], qr_opacity))
    scale = width / qr_img.size[0]
    border_px = int(border * box_size * scale)
    qr_w = width - (border_px * 2)
    qr_h = height - (border_px * 2)
    qr_scaled = qr_img.resize((qr_w, qr_h))
    qr_layer = Image.new("RGBA", (width, height), color=(0, 0, 0, 0))
    qr_box = (
        border_px, # from left
        border_px, # from top
        height - border_px, # from bottom
        width - border_px # from right
    )
    qr_layer.paste(qr_scaled, qr_box)
    return qr_layer

def make_pretty_qr(data, img, border=4, qr_opacity=200, qr_rgb=(0,0,0)):
    """
    Create an image from a QR encoding of the provided data, composited with
   the provided image file (which must be square) optimising for scannability.
    """
    base_img = load_image(img)
    qr_img = make_qr(data, 1, border, base_img.size[0], base_img.size[1], qr_opacity=qr_opacity, qr_rgb=qr_rgb)
    return Image.alpha_composite(base_img, qr_img)
    



class PrettyException(Exception):
    "Semantic errors in prettyqr."
    pass

class ImageNotSquareException(PrettyException):
    pass

class BaseImageSmallerThanQRCodeException(PrettyException):
    pass



"""
prettyqr

https://github.com/olorin/prettyqr
"""

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    desc = f.read()

setup(
    name="prettyqr",
    version="0.0.1",
    description="Create QR codes merged with images.",
    long_description=desc,
    url="https://github.com/olorin/prettyqr",
    author="Sharif Olorin",
    author_email="sio@tesser.org",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Multimedia :: Graphics",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
    ],
    keywords="qrcode image",
    packages=find_packages(exclude=["tests"]),
    install_requires=[
        "pillow>=3.2",
        "qrcode>=5.3",
    ],
    entry_points={
        "console_scripts": [
            "prettyqr=prettyqr.cli:main",
        ],
    },
    data_files=[
        ("img", ["img/dna-orig.png", "img/dna-qr.png"]),
    ],
)



import sys
import argparse

from .pretty import make_pretty_qr

def mkparser():
    parser = argparse.ArgumentParser(description="Make pretty QR codes.")
    parser.add_argument(
        "data",
        metavar="DATA",
        type=str,
        help="Data to encode (e.g., \"http://example.com\")."
    )
    parser.add_argument(
        "image",
        metavar="IMAGE-FILE",
        type=str,
        help="Image file to composite (e.g., \"lena.jpg\")."
    )
    parser.add_argument(
        "--border",
        dest="border",
        type=int,
        default=4,
        help="Size of border (in multiples of QR module size). Defaults to 4. Can be zero if the QR code is intended for printing or use on a light background."
    )
    parser.add_argument(
        "--out-file",
        dest="out_file",
        type=str,
        default="prettyqr.png",
        help="Path to write composite image file to. Extension will determine the output format. Defaults to \"prettyqr.png\"."
    )
    parser.add_argument(
        "--qr-opacity",
        dest="qr_opacity",
        type=int,
        default=200,
        help="Opacity of the QR code modules, between 0 (invisible) and 255 (opaque). Default is 200."
    )
    parser.add_argument(
        "--qr-red",
        dest="qr_red",
        type=int,
        default=0,
        help="Red element of the QR module color, between 0 and 255. Default is 0.",
    )
    parser.add_argument(
        "--qr-green",
        dest="qr_green",
        type=int,
        default=0,
        help="Green element of the QR module color, between 0 and 255. Default is 0.",
    )
    parser.add_argument(
        "--qr-blue",
        dest="qr_blue",
        type=int,
        default=0,
        help="Blue element of the QR module color, between 0 and 255. Default is 0.",
    )


    return parser

def main():
    args = mkparser().parse_args()
    qr = make_pretty_qr(
        args.data,
        args.image,
        border=args.border,
        qr_opacity=args.qr_opacity,
        qr_rgb=(args.qr_red, args.qr_green, args.qr_blue),
    )
    qr.save(args.out_file)
